@@ -72,8 +72,9 @@ struct RawToken {
     type = RawTokenType::STR;
     return *this;
   }
-  RawToken& operator=(const double v) {
-    dvalue = v;
+  RawToken& operator=(const std::pair<double, std::string> vs) {
+    dvalue = vs.first;
+    svalue = vs.second;
     type = RawTokenType::CONS;
     return *this;
   }
@@ -309,23 +310,6 @@ Model Reader::read() {
   splittokens();
 
   // std::clog << "Setting up model..." << std::endl;
-  //
-  // Since
-  //
-  // "The problem statement must begin with the word MINIMIZE or
-  // MAXIMIZE, MINIMUM or MAXIMUM, or the abbreviations MIN or MAX, in
-  // any combination of upper- and lower-case characters. The word
-  // introduces the objective function section."
-  //
-  // Use positivity of sectiontokens.count(LpSectionKeyword::OBJMIN) +
-  // sectiontokens.count(LpSectionKeyword::OBJMAX) to identify garbage file
-  //
-
-  const int num_objective_section =
-    sectiontokens.count(LpSectionKeyword::OBJMIN) +
-    sectiontokens.count(LpSectionKeyword::OBJMAX);
-  lpassert(num_objective_section>0);
-
   processsections();
   processedtokens.clear();
   processedtokens.shrink_to_fit();
@@ -357,8 +341,6 @@ void Reader::parseexpression(std::vector<ProcessedToken>::iterator& it,
           std::shared_ptr<LinTerm>(new LinTerm());
       linterm->coef = it->value;
       linterm->var = builder.getvarbyname(name);
-      //	 Rprintf("LpReader: Term   %+g %s\n", linterm->coef,
-      //name.c_str());
       expr->linterms.push_back(linterm);
 
       ++it;
@@ -368,7 +350,6 @@ void Reader::parseexpression(std::vector<ProcessedToken>::iterator& it,
 
     // const
     if (it->type == ProcessedTokenType::CONST) {
-      //      Rprintf("LpReader: Offset change from %+g by %+g\n", expr->offset, it->value);
       expr->offset += it->value;
       ++it;
       continue;
@@ -382,8 +363,6 @@ void Reader::parseexpression(std::vector<ProcessedToken>::iterator& it,
           std::shared_ptr<LinTerm>(new LinTerm());
       linterm->coef = 1.0;
       linterm->var = builder.getvarbyname(name);
-      //	 Rprintf("LpReader: Term   %+g %s\n", linterm->coef,
-      //name.c_str());
       expr->linterms.push_back(linterm);
 
       ++it;
@@ -972,6 +951,15 @@ void Reader::processtokens() {
       continue;
     }
 
+    // constraint identifier - with numeric constant value as name?
+    if (rawtokens[0].istype(RawTokenType::CONS) &&
+        rawtokens[1].istype(RawTokenType::COLON)) {
+      processedtokens.emplace_back(ProcessedTokenType::CONID,
+                                   rawtokens[0].svalue);
+      nextrawtoken(2);
+      continue;
+    }
+
     // check if free
     if (rawtokens[0].istype(RawTokenType::STR) &&
         iskeyword(svalue_lc, LP_KEYWORD_FREE, LP_KEYWORD_FREE_N)) {
@@ -1202,7 +1190,6 @@ bool Reader::readnexttoken(RawToken& t) {
 
   // check single character tokens
   char nextchar = this->linebuffer[this->linebufferpos];
-
   switch (nextchar) {
     // check for comment
     case '\\':
@@ -1298,7 +1285,12 @@ bool Reader::readnexttoken(RawToken& t) {
   char* endptr;
   double constant = strtod(startptr, &endptr);
   if (endptr != startptr) {
-    t = constant;
+    // Extract the string corresponding to the double, in case the
+    // double is a constraint name
+    size_t double_len = endptr - startptr;
+    std::string double_name = this->linebuffer.substr(this->linebufferpos, double_len);
+    // t = constant;
+    t = std::make_pair(constant, double_name);
     this->linebufferpos += endptr - startptr;
     return true;
   }
