@@ -62,6 +62,9 @@ HighsInt Highs_lpCall(const HighsInt num_col, const HighsInt num_row,
       if (copy_row_basis) row_basis_status[i] = (HighsInt)basis.row_status[i];
     }
   }
+
+  highs.resetGlobalScheduler(true);
+
   return (HighsInt)status;
 }
 
@@ -105,6 +108,8 @@ HighsInt Highs_mipCall(const HighsInt num_col, const HighsInt num_row,
     }
   }
 
+  highs.resetGlobalScheduler(true);
+
   return (HighsInt)status;
 }
 
@@ -161,12 +166,18 @@ HighsInt Highs_qpCall(
       if (copy_row_basis) row_basis_status[i] = (HighsInt)basis.row_status[i];
     }
   }
+
+  highs.resetGlobalScheduler(true);
+
   return (HighsInt)status;
 }
 
 void* Highs_create(void) { return new Highs(); }
 
-void Highs_destroy(void* highs) { delete (Highs*)highs; }
+void Highs_destroy(void* highs) {
+  Highs::resetGlobalScheduler(true);
+  delete (Highs*)highs;
+}
 
 const char* Highs_version(void) { return highsVersion(); }
 HighsInt Highs_versionMajor(void) { return highsVersionMajor(); }
@@ -929,6 +940,14 @@ HighsInt Highs_changeRowBounds(void* highs, const HighsInt row,
   return (HighsInt)((Highs*)highs)->changeRowBounds(row, lower, upper);
 }
 
+HighsInt Highs_changeRowsBoundsByRange(void* highs, const HighsInt from_row,
+                                       const HighsInt to_row,
+                                       const double* lower,
+                                       const double* upper) {
+  return (HighsInt)((Highs*)highs)
+      ->changeRowsBounds(from_row, to_row, lower, upper);
+}
+
 HighsInt Highs_changeRowsBoundsBySet(void* highs,
                                      const HighsInt num_set_entries,
                                      const HighsInt* set, const double* lower,
@@ -1163,14 +1182,14 @@ HighsInt Highs_getPresolvedNumNz(const void* highs) {
 
 // Gets pointers to all the public data members of HighsLp: avoids
 // duplicate code in Highs_getModel, Highs_getPresolvedLp,
-HighsInt Highs_getHighsLpData(const HighsLp& lp, const HighsInt a_format,
-                              HighsInt* num_col, HighsInt* num_row,
-                              HighsInt* num_nz, HighsInt* sense, double* offset,
-                              double* col_cost, double* col_lower,
-                              double* col_upper, double* row_lower,
-                              double* row_upper, HighsInt* a_start,
-                              HighsInt* a_index, double* a_value,
-                              HighsInt* integrality) {
+static HighsInt Highs_getHighsLpData(const HighsLp& lp, const HighsInt a_format,
+                                     HighsInt* num_col, HighsInt* num_row,
+                                     HighsInt* num_nz, HighsInt* sense,
+                                     double* offset, double* col_cost,
+                                     double* col_lower, double* col_upper,
+                                     double* row_lower, double* row_upper,
+                                     HighsInt* a_start, HighsInt* a_index,
+                                     double* a_value, HighsInt* integrality) {
   const MatrixFormat desired_a_format =
       a_format == HighsInt(MatrixFormat::kColwise) ? MatrixFormat::kColwise
                                                    : MatrixFormat::kRowwise;
@@ -1278,6 +1297,23 @@ HighsInt Highs_getLp(const void* highs, const HighsInt a_format,
                               a_start, a_index, a_value, integrality);
 }
 
+HighsInt Highs_getFixedLp(const void* highs, const HighsInt a_format,
+                          HighsInt* num_col, HighsInt* num_row,
+                          HighsInt* num_nz, HighsInt* sense, double* offset,
+                          double* col_cost, double* col_lower,
+                          double* col_upper, double* row_lower,
+                          double* row_upper, HighsInt* a_start,
+                          HighsInt* a_index, double* a_value) {
+  HighsLp lp;
+  HighsInt status = HighsInt(((Highs*)highs)->getFixedLp(lp));
+  if (status == kHighsStatusError) return status;
+  HighsInt* integrality = nullptr;
+  Highs_getHighsLpData(lp, a_format, num_col, num_row, num_nz, sense, offset,
+                       col_cost, col_lower, col_upper, row_lower, row_upper,
+                       a_start, a_index, a_value, integrality);
+  return status;
+}
+
 HighsInt Highs_getPresolvedLp(const void* highs, const HighsInt a_format,
                               HighsInt* num_col, HighsInt* num_row,
                               HighsInt* num_nz, HighsInt* sense, double* offset,
@@ -1292,6 +1328,62 @@ HighsInt Highs_getPresolvedLp(const void* highs, const HighsInt a_format,
                               a_start, a_index, a_value, integrality);
 }
 
+HighsInt Highs_getIis(void* highs, HighsInt* iis_num_col, HighsInt* iis_num_row,
+                      HighsInt* col_index, HighsInt* row_index,
+                      HighsInt* col_bound, HighsInt* row_bound,
+                      HighsInt* col_status, HighsInt* row_status) {
+  HighsIis iis;
+  HighsInt status = (HighsInt)((Highs*)highs)->getIis(iis);
+  if (status == (HighsInt)HighsStatus::kError) return status;
+  *iis_num_col = iis.col_index_.size();
+  *iis_num_row = iis.row_index_.size();
+  if (col_index != nullptr) {
+    for (size_t i = 0; i < static_cast<size_t>(*iis_num_col); i++) {
+      col_index[i] = iis.col_index_[i];
+    }
+  }
+  if (row_index != nullptr) {
+    for (size_t i = 0; i < static_cast<size_t>(*iis_num_row); i++) {
+      row_index[i] = iis.row_index_[i];
+    }
+  }
+  if (col_bound != nullptr) {
+    for (size_t i = 0; i < static_cast<size_t>(*iis_num_col); i++) {
+      col_bound[i] = iis.col_bound_[i];
+    }
+  }
+  if (row_bound != nullptr) {
+    for (size_t i = 0; i < static_cast<size_t>(*iis_num_row); i++) {
+      row_bound[i] = iis.row_bound_[i];
+    }
+  }
+  if (col_status != nullptr) {
+    for (size_t i = 0;
+         i < static_cast<size_t>(((Highs*)highs)->getLp().num_col_); i++) {
+      col_status[i] = iis.col_status_[i];
+    }
+  }
+  if (row_status != nullptr) {
+    for (size_t i = 0;
+         i < static_cast<size_t>(((Highs*)highs)->getLp().num_row_); i++) {
+      row_status[i] = iis.row_status_[i];
+    }
+  }
+  return status;
+}
+
+HighsInt Highs_getIisLp(const void* highs, const HighsInt a_format,
+                        HighsInt* num_col, HighsInt* num_row, HighsInt* num_nz,
+                        HighsInt* sense, double* offset, double* col_cost,
+                        double* col_lower, double* col_upper, double* row_lower,
+                        double* row_upper, HighsInt* a_start, HighsInt* a_index,
+                        double* a_value, HighsInt* integrality) {
+  return Highs_getHighsLpData(((Highs*)highs)->getIisLp(), a_format, num_col,
+                              num_row, num_nz, sense, offset, col_cost,
+                              col_lower, col_upper, row_lower, row_upper,
+                              a_start, a_index, a_value, integrality);
+}
+
 HighsInt Highs_crossover(void* highs, const HighsInt num_col,
                          const HighsInt num_row, const double* col_value,
                          const double* col_dual, const double* row_dual) {
@@ -1486,6 +1578,34 @@ const void* Highs_getCallbackDataOutItem(const HighsCallbackDataOut* data_out,
   return nullptr;
 }
 
+HighsInt Highs_setCallbackSolution(HighsCallbackDataIn* data_in,
+                                   HighsInt num_entries, const double* value) {
+  if (data_in != nullptr && data_in->cbdata != nullptr) {
+    HighsCallbackInput* obj = static_cast<HighsCallbackInput*>(data_in->cbdata);
+    return static_cast<int>(obj->setSolution(num_entries, value));
+  } else
+    return static_cast<int>(HighsStatus::kError);
+}
+
+HighsInt Highs_setCallbackSparseSolution(HighsCallbackDataIn* data_in,
+                                         HighsInt num_entries,
+                                         const HighsInt* index,
+                                         const double* value) {
+  if (data_in != nullptr && data_in->cbdata != nullptr) {
+    HighsCallbackInput* obj = static_cast<HighsCallbackInput*>(data_in->cbdata);
+    return static_cast<int>(obj->setSolution(num_entries, index, value));
+  } else
+    return static_cast<int>(HighsStatus::kError);
+}
+
+HighsInt Highs_repairCallbackSolution(HighsCallbackDataIn* data_in) {
+  if (data_in != nullptr && data_in->cbdata != nullptr) {
+    HighsCallbackInput* obj = static_cast<HighsCallbackInput*>(data_in->cbdata);
+    return static_cast<int>(obj->repairSolution());
+  } else
+    return static_cast<int>(HighsStatus::kError);
+}
+
 // *********************
 // * Deprecated methods*
 // *********************
