@@ -20,12 +20,10 @@
 #include "../extern/filereaderlp/reader.hpp"
 #include "lp_data/HighsLpUtils.h"
 
-const bool original_double_format = false;
-const bool allow_model_names = true;
-
 FilereaderRetcode FilereaderLp::readModelFromFile(const HighsOptions& options,
                                                   const std::string filename,
                                                   HighsModel& model) {
+  bool warning_issued = false;
   HighsLp& lp = model.lp_;
   HighsHessian& hessian = model.hessian_;
   try {
@@ -71,8 +69,7 @@ FilereaderRetcode FilereaderLp::readModelFromFile(const HighsOptions& options,
     //
     lp.offset_ = m.objective->offset;
     lp.col_cost_.resize(lp.num_col_, 0.0);
-    for (size_t i = 0; i < m.objective->linterms.size(); i++) {
-      std::shared_ptr<LinTerm> lt = m.objective->linterms[i];
+    for (const auto& lt : m.objective->linterms) {
       lp.col_cost_[varindex[lt->var->name]] = lt->coef;
     }
 
@@ -173,27 +170,126 @@ FilereaderRetcode FilereaderLp::readModelFromFile(const HighsOptions& options,
                    "with same prefix: row names cleared\n");
     }
 
-    HighsInt nz = 0;
+    HighsInt num_nz = 0;
     // lp.a_matrix_ is initialised with start_[0] for fictitious
     // column 0, so have to clear this before pushing back start
     lp.a_matrix_.start_.clear();
     assert((int)lp.a_matrix_.start_.size() == 0);
-    for (HighsInt i = 0; i < lp.num_col_; i++) {
-      std::shared_ptr<Variable> var = m.variables[i];
-      lp.a_matrix_.start_.push_back(nz);
+    for (const auto& var : m.variables) {
+      lp.a_matrix_.start_.push_back(num_nz);
       for (size_t j = 0; j < consofvarmap_index[var].size(); j++) {
         double value = consofvarmap_value[var][j];
         if (value) {
           lp.a_matrix_.index_.push_back(consofvarmap_index[var][j]);
           lp.a_matrix_.value_.push_back(value);
-          nz++;
+          num_nz++;
         }
       }
     }
-    lp.a_matrix_.start_.push_back(nz);
+    lp.a_matrix_.start_.push_back(num_nz);
     lp.a_matrix_.format_ = MatrixFormat::kColwise;
     lp.sense_ = m.sense == ObjectiveSense::MIN ? ObjSense::kMinimize
                                                : ObjSense::kMaximize;
+    // In a .lp file, more than one term involving the same variable
+    // can appear in a constraint, resulting in repeated row indices
+    // in a column
+    HighsSparseMatrix& matrix = lp.a_matrix_;
+    assert(matrix.isColwise());
+    num_nz = 0;
+    std::vector<double> column(lp.num_row_, 0);
+    std::vector<HighsInt> nz_count(lp.num_row_, 0);
+    std::vector<HighsInt> zero_count(lp.num_row_, 0);
+    HighsInt sum_num_duplicate = 0;
+    HighsInt sum_num_zero = 0;
+    HighsInt sum_cancellation = 0;
+    HighsInt num_report = 0;
+    HighsInt max_num_report = 10;
+    for (HighsInt iCol = 0; iCol < lp.num_col_; iCol++) {
+      // Save the start, since this will be reduced if there are
+      // repeated row indices in a column
+      const HighsInt from_el = matrix.start_[iCol];
+      for (HighsInt iEl = from_el; iEl < matrix.start_[iCol + 1]; iEl++) {
+        // Add in the value to zero or any previous nonzero in this
+        // row
+        HighsInt iRow = matrix.index_[iEl];
+        double value = matrix.value_[iEl];
+        if (value) {
+          column[iRow] += value;
+          nz_count[iRow]++;
+        } else {
+          zero_count[iRow]++;
+        }
+      }
+      // Pass through the column again, storing and then zeroing the
+      // entries in column - both to eliminate the duplicate and
+      // ensure that column is zeroed for the next matrix column.
+      matrix.start_[iCol] = num_nz;
+      for (HighsInt iEl = from_el; iEl < matrix.start_[iCol + 1]; iEl++) {
+        HighsInt iRow = matrix.index_[iEl];
+        if (column[iRow]) {
+          assert(num_nz <= iEl);
+          matrix.index_[num_nz] = iRow;
+          matrix.value_[num_nz] = column[iRow];
+          num_nz++;
+        }
+        // Report explicit zeros and/or sum to a nonzero value
+        HighsInt num_ocurrence = zero_count[iRow] + nz_count[iRow];
+        if (num_ocurrence > 1) {
+          if (nz_count[iRow] > 1) {
+            if (num_report < max_num_report)
+              highsLogUser(options.log_options, HighsLogType::kWarning,
+                           "Column %d (name \"%s\") occurs %d times in row %d "
+                           "(name \"%s\"): values summed to %g\n",
+                           int(iCol), lp.col_names_[iCol].c_str(),
+                           int(num_ocurrence), int(iRow),
+                           lp.row_names_[iRow].c_str(), column[iRow]);
+            num_report++;
+          }
+          if (zero_count[iRow] > 0) {
+            if (num_report < max_num_report)
+              highsLogUser(options.log_options, HighsLogType::kWarning,
+                           "Column %d (name \"%s\") contains %d explicit zero "
+                           "coefficient%s in row %d (name \"%s\")\n",
+                           int(iCol), lp.col_names_[iCol].c_str(),
+                           int(zero_count[iRow]),
+                           zero_count[iRow] > 1 ? "s" : "", int(iRow),
+                           lp.row_names_[iRow].c_str());
+            num_report++;
+          }
+          sum_num_duplicate += (num_ocurrence - 1);
+          sum_num_zero += zero_count[iRow];
+          if (column[iRow] == 0 && nz_count[iRow] > 0) sum_cancellation++;
+        }
+        zero_count[iRow] = 0;
+        nz_count[iRow] = 0;
+        column[iRow] = 0;
+      }
+      for (HighsInt iRow = 0; iRow < lp.num_row_; iRow++) {
+        assert(zero_count[iRow] == 0);
+        assert(nz_count[iRow] == 0);
+        assert(column[iRow] == 0);
+      }
+    }
+    matrix.start_[lp.num_col_] = num_nz;
+    warning_issued = sum_num_duplicate > 0 || sum_num_zero > 0;
+    HighsInt num_report_skipped = num_report - max_num_report;
+    if (num_report_skipped > 0)
+      highsLogUser(options.log_options, HighsLogType::kInfo,
+                   "Skipped %d further warning%s of this kind\n",
+                   int(num_report_skipped), num_report_skipped > 1 ? "s" : "");
+
+    if (sum_num_duplicate > 0)
+      highsLogUser(options.log_options, HighsLogType::kWarning,
+                   "lp file contains %d repeated variable%s in constraints: "
+                   "summing them yielded %d cancellation%s\n",
+                   int(sum_num_duplicate), sum_num_duplicate > 1 ? "s" : "",
+                   int(sum_cancellation),
+                   (sum_cancellation == 0 || sum_cancellation > 1) ? "s" : "");
+    if (sum_num_zero > 0)
+      highsLogUser(options.log_options, HighsLogType::kWarning,
+                   "lp file contains %d explicit zero%s\n", int(sum_num_zero),
+                   sum_num_zero > 1 ? "s" : "");
+
   } catch (std::invalid_argument& ex) {
     // lpassert in extern/filereaderlp/def.hpp throws
     // std::invalid_argument whatever the error. Hence, unless
@@ -208,7 +304,7 @@ FilereaderRetcode FilereaderLp::readModelFromFile(const HighsOptions& options,
     return FilereaderRetcode::kParserError;
   }
   lp.ensureColwise();
-  return FilereaderRetcode::kOk;
+  return warning_issued ? FilereaderRetcode::kWarning : FilereaderRetcode::kOk;
 }
 
 void FilereaderLp::writeToFile(FILE* file, const char* format, ...) {
@@ -228,53 +324,36 @@ void FilereaderLp::writeToFile(FILE* file, const char* format, ...) {
   }
 }
 
-void FilereaderLp::writeToFileLineend(FILE* file) {
+void FilereaderLp::writeToFileLineEnd(FILE* file) {
   fprintf(file, "\n");
   this->linelength = 0;
 }
 
 void FilereaderLp::writeToFileValue(FILE* file, const double value,
                                     const bool force_plus) {
-  if (original_double_format) {
-    this->writeToFile(file, " %+g", value);
+  // As for writeModelAsMps
+  if (force_plus) {
+    this->writeToFile(file, " %+.15g", value);
   } else {
-    // As for writeModelAsMps
-    if (force_plus) {
-      this->writeToFile(file, " %+.15g", value);
-    } else {
-      this->writeToFile(file, " %.15g", value);
-    }
+    this->writeToFile(file, " %.15g", value);
   }
 }
 
-void FilereaderLp::writeToFileVar(FILE* file, const HighsInt var_index) {
-  this->writeToFile(file, " x%" HIGHSINT_FORMAT, var_index + 1);
-}
-
 void FilereaderLp::writeToFileVar(FILE* file, const std::string var_name) {
   this->writeToFile(file, " %s", var_name.c_str());
 }
 
-void FilereaderLp::writeToFileCon(FILE* file, const HighsInt con_index) {
-  this->writeToFile(file, " con%" HIGHSINT_FORMAT, con_index + 1);
-}
-
 void FilereaderLp::writeToFileMatrixRow(FILE* file, const HighsInt iRow,
-                                        const HighsSparseMatrix ar_matrix,
-                                        const std::vector<string> col_names) {
+                                        const HighsSparseMatrix& ar_matrix,
+                                        const std::vector<string>& col_names) {
   assert(ar_matrix.isRowwise());
-  const bool has_col_names = allow_model_names && col_names.size() > 0;
 
   for (HighsInt iEl = ar_matrix.start_[iRow]; iEl < ar_matrix.start_[iRow + 1];
        iEl++) {
     HighsInt iCol = ar_matrix.index_[iEl];
     double coef = ar_matrix.value_[iEl];
     this->writeToFileValue(file, coef);
-    if (has_col_names) {
-      this->writeToFileVar(file, col_names[iCol]);
-    } else {
-      this->writeToFileVar(file, iCol);
-    }
+    this->writeToFileVar(file, col_names[iCol]);
   }
 }
 
@@ -282,36 +361,31 @@ HighsStatus FilereaderLp::writeModelToFile(const HighsOptions& options,
                                            const std::string filename,
                                            const HighsModel& model) {
   const HighsLp& lp = model.lp_;
+
+  const bool ok_names = lp.okNames();
+  assert(ok_names);
+  if (!ok_names) return HighsStatus::kError;
+
   // Create a row-wise copy of the matrix
   HighsSparseMatrix ar_matrix = lp.a_matrix_;
   ar_matrix.ensureRowwise();
 
-  const bool has_col_names =
-      allow_model_names &&
-      lp.col_names_.size() == static_cast<size_t>(lp.num_col_);
-  const bool has_row_names =
-      allow_model_names &&
-      lp.row_names_.size() == static_cast<size_t>(lp.num_row_);
   FILE* file = fopen(filename.c_str(), "w");
 
   // write comment at the start of the file
   this->writeToFile(file, "\\ %s", LP_COMMENT_FILESTART);
-  this->writeToFileLineend(file);
+  this->writeToFileLineEnd(file);
 
   // write objective
   this->writeToFile(file, "%s",
                     lp.sense_ == ObjSense::kMinimize ? "min" : "max");
-  this->writeToFileLineend(file);
+  this->writeToFileLineEnd(file);
   this->writeToFile(file, " obj:");
   for (HighsInt iCol = 0; iCol < lp.num_col_; iCol++) {
     double coef = lp.col_cost_[iCol];
     if (coef != 0.0) {
       this->writeToFileValue(file, coef);
-      if (has_col_names) {
-        this->writeToFileVar(file, lp.col_names_[iCol]);
-      } else {
-        this->writeToFileVar(file, iCol);
-      }
+      this->writeToFileVar(file, lp.col_names_[iCol]);
     }
   }
   this->writeToFile(file,
@@ -327,15 +401,9 @@ HighsStatus FilereaderLp::writeModelToFile(const HighsOptions& options,
           if (iCol != iRow) coef *= 2;
           if (coef != 0.0) {
             this->writeToFileValue(file, coef);
-            if (has_col_names) {
-              this->writeToFileVar(file, lp.col_names_[iCol]);
-              this->writeToFile(file, " *");
-              this->writeToFileVar(file, lp.col_names_[iRow]);
-            } else {
-              this->writeToFileVar(file, iCol);
-              this->writeToFile(file, " *");
-              this->writeToFileVar(file, iRow);
-            }
+            this->writeToFileVar(file, lp.col_names_[iCol]);
+            this->writeToFile(file, " *");
+            this->writeToFileVar(file, lp.col_names_[iRow]);
           }
         }
       }
@@ -345,25 +413,21 @@ HighsStatus FilereaderLp::writeModelToFile(const HighsOptions& options,
   }
   double coef = lp.offset_;
   if (coef != 0) this->writeToFileValue(file, coef);
-  this->writeToFileLineend(file);
+  this->writeToFileLineEnd(file);
 
   // write constraint section, lower & upper bounds are one constraint
   // each
   this->writeToFile(file, "st");
-  this->writeToFileLineend(file);
+  this->writeToFileLineEnd(file);
   for (HighsInt iRow = 0; iRow < lp.num_row_; iRow++) {
     if (lp.row_lower_[iRow] == lp.row_upper_[iRow]) {
       // Equality constraint
-      if (has_row_names) {
-        this->writeToFileVar(file, lp.row_names_[iRow]);
-      } else {
-        this->writeToFileCon(file, iRow);
-      }
+      this->writeToFileVar(file, lp.row_names_[iRow]);
       this->writeToFile(file, ":");
       this->writeToFileMatrixRow(file, iRow, ar_matrix, lp.col_names_);
       this->writeToFile(file, " =");
       this->writeToFileValue(file, lp.row_lower_[iRow], true);
-      this->writeToFileLineend(file);
+      this->writeToFileLineEnd(file);
     } else {
       // Need to distinguish the names when writing out boxed
       // constraint row as two single-sided constraints
@@ -371,11 +435,7 @@ HighsStatus FilereaderLp::writeModelToFile(const HighsOptions& options,
           lp.row_lower_[iRow] > -kHighsInf && lp.row_upper_[iRow] < kHighsInf;
       if (lp.row_lower_[iRow] > -kHighsInf) {
         // Has a lower bound
-        if (has_row_names) {
-          this->writeToFileVar(file, lp.row_names_[iRow]);
-        } else {
-          this->writeToFileCon(file, iRow);
-        }
+        this->writeToFileVar(file, lp.row_names_[iRow]);
         if (boxed) {
           this->writeToFile(file, "lo:");
         } else {
@@ -384,15 +444,11 @@ HighsStatus FilereaderLp::writeModelToFile(const HighsOptions& options,
         this->writeToFileMatrixRow(file, iRow, ar_matrix, lp.col_names_);
         this->writeToFile(file, " >=");
         this->writeToFileValue(file, lp.row_lower_[iRow], true);
-        this->writeToFileLineend(file);
+        this->writeToFileLineEnd(file);
       }
       if (lp.row_upper_[iRow] < kHighsInf) {
         // Has an upper bound
-        if (has_row_names) {
-          this->writeToFileVar(file, lp.row_names_[iRow]);
-        } else {
-          this->writeToFileCon(file, iRow);
-        }
+        this->writeToFileVar(file, lp.row_names_[iRow]);
         if (boxed) {
           this->writeToFile(file, "up:");
         } else {
@@ -401,33 +457,25 @@ HighsStatus FilereaderLp::writeModelToFile(const HighsOptions& options,
         this->writeToFileMatrixRow(file, iRow, ar_matrix, lp.col_names_);
         this->writeToFile(file, " <=");
         this->writeToFileValue(file, lp.row_upper_[iRow], true);
-        this->writeToFileLineend(file);
+        this->writeToFileLineEnd(file);
       }
     }
   }
 
   // write bounds section
   this->writeToFile(file, "bounds");
-  this->writeToFileLineend(file);
+  this->writeToFileLineEnd(file);
   for (HighsInt iCol = 0; iCol < lp.num_col_; iCol++) {
     const bool default_bounds =
         lp.col_lower_[iCol] == 0 && lp.col_upper_[iCol] == kHighsInf;
     if (default_bounds) continue;
     if (lp.col_lower_[iCol] <= -kHighsInf && lp.col_upper_[iCol] >= kHighsInf) {
       // Free variable
-      if (has_col_names) {
-        this->writeToFileVar(file, lp.col_names_[iCol]);
-      } else {
-        this->writeToFileVar(file, iCol);
-      }
+      this->writeToFileVar(file, lp.col_names_[iCol]);
       this->writeToFile(file, " free");
     } else if (lp.col_lower_[iCol] == lp.col_upper_[iCol]) {
       // Fixed variable
-      if (has_col_names) {
-        this->writeToFileVar(file, lp.col_names_[iCol]);
-      } else {
-        this->writeToFileVar(file, iCol);
-      }
+      this->writeToFileVar(file, lp.col_names_[iCol]);
       this->writeToFile(file, " =");
       this->writeToFileValue(file, lp.col_upper_[iCol], false);
     } else {
@@ -438,70 +486,54 @@ HighsStatus FilereaderLp::writeModelToFile(const HighsOptions& options,
         this->writeToFileValue(file, lp.col_lower_[iCol], false);
         this->writeToFile(file, " <=");
       }
-      if (has_col_names) {
-        this->writeToFileVar(file, lp.col_names_[iCol]);
-      } else {
-        this->writeToFileVar(file, iCol);
-      }
+      this->writeToFileVar(file, lp.col_names_[iCol]);
       if (lp.col_upper_[iCol] < kHighsInf) {
         // Finite upper bound
         this->writeToFile(file, " <=");
         this->writeToFileValue(file, lp.col_upper_[iCol], false);
       }
     }
-    this->writeToFileLineend(file);
+    this->writeToFileLineEnd(file);
   }
   if (lp.integrality_.size() > 0) {
     // write binary section
     this->writeToFile(file, "bin");
-    this->writeToFileLineend(file);
+    this->writeToFileLineEnd(file);
     for (HighsInt iCol = 0; iCol < lp.num_col_; iCol++) {
       if (lp.integrality_[iCol] == HighsVarType::kInteger) {
         if (lp.col_lower_[iCol] == 0.0 && lp.col_upper_[iCol] == 1.0) {
-          if (has_col_names) {
-            this->writeToFileVar(file, lp.col_names_[iCol]);
-          } else {
-            this->writeToFileVar(file, iCol);
-          }
-          this->writeToFileLineend(file);
+          this->writeToFileVar(file, lp.col_names_[iCol]);
+          this->writeToFileLineEnd(file);
         }
       }
     }
 
     // write general section
     this->writeToFile(file, "gen");
-    this->writeToFileLineend(file);
+    this->writeToFileLineEnd(file);
     for (HighsInt iCol = 0; iCol < lp.num_col_; iCol++) {
       if (lp.integrality_[iCol] == HighsVarType::kInteger) {
         if (lp.col_lower_[iCol] != 0.0 || lp.col_upper_[iCol] != 1.0) {
-          if (has_col_names) {
-            this->writeToFileVar(file, lp.col_names_[iCol]);
-          } else {
-            this->writeToFileVar(file, iCol);
-          }
-          this->writeToFileLineend(file);
+          this->writeToFileVar(file, lp.col_names_[iCol]);
+          this->writeToFileLineEnd(file);
         }
       }
     }
 
     // write semi section
     this->writeToFile(file, "semi");
-    this->writeToFileLineend(file);
+    this->writeToFileLineEnd(file);
     for (HighsInt iCol = 0; iCol < lp.num_col_; iCol++) {
       if (lp.integrality_[iCol] == HighsVarType::kSemiContinuous ||
           lp.integrality_[iCol] == HighsVarType::kSemiInteger) {
-        if (has_col_names) {
-          this->writeToFileVar(file, lp.col_names_[iCol]);
-        } else {
-          this->writeToFileVar(file, iCol);
-        }
-        this->writeToFileLineend(file);
+        this->writeToFileVar(file, lp.col_names_[iCol]);
+        this->writeToFileLineEnd(file);
       }
     }
   }
   // write end
   this->writeToFile(file, "end");
-  this->writeToFileLineend(file);
+  this->writeToFileLineEnd(file);
 
   fclose(file);
   return HighsStatus::kOk;
