@@ -1,4 +1,3 @@
-#include <Rcpp.h>
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 /*                                                                       */
 /*    This file is part of the HiGHS linear optimization suite           */
@@ -26,19 +25,22 @@ void highsOpenLogFile(HighsLogOptions& log_options,
   OptionStatus status =
       getOptionIndex(log_options, "log_file", option_records, index);
   assert(status == OptionStatus::kOk);
-  if (log_options.log_stream != NULL) {
+  if (log_options.log_stream != nullptr) {
     // Current log file stream is not null, so flush and close it
     fflush(log_options.log_stream);
     fclose(log_options.log_stream);
+    // Set the stream to null to give a test whether it has been
+    // closed (and avoid trying to close it again which causes an
+    // error)
+    log_options.log_stream = nullptr;
   }
-  if (log_file.compare("")) {
-    // New log file name is not empty, so open it, appending if
-    // possible
+  assert(!log_options.log_stream);
+  // If new log file name is not empty, open it, appending if possible
+  //
+  // If fopen fails then it returns nullptr, so log_file is open for
+  // writing or nullptr
+  if (log_file.compare(""))
     log_options.log_stream = fopen(log_file.c_str(), "a");
-  } else {
-    // New log file name is empty, so set the stream to null
-    log_options.log_stream = NULL;
-  }
   OptionRecordString& option = *(OptionRecordString*)option_records[index];
   option.assignvalue(log_file);
 }
@@ -55,40 +57,131 @@ static std::string optionEntryTypeToString(const HighsOptionType type) {
   }
 }
 
-bool commandLineOffChooseOnOk(const HighsLogOptions& report_log_options,
-                              const string& name, const string& value) {
+bool optionOffChooseOnOk(const HighsLogOptions& report_log_options,
+                         const string& name, const string& value) {
   if (value == kHighsOffString || value == kHighsChooseString ||
       value == kHighsOnString)
     return true;
   highsLogUser(
-      report_log_options, HighsLogType::kWarning,
+      report_log_options, HighsLogType::kError,
       "Value \"%s\" for %s option is not one of \"%s\", \"%s\" or \"%s\"\n",
       value.c_str(), name.c_str(), kHighsOffString.c_str(),
       kHighsChooseString.c_str(), kHighsOnString.c_str());
   return false;
 }
 
-bool commandLineOffOnOk(const HighsLogOptions& report_log_options,
-                        const string& name, const string& value) {
+bool optionOffOnOk(const HighsLogOptions& report_log_options,
+                   const string& name, const string& value) {
   if (value == kHighsOffString || value == kHighsOnString) return true;
-  highsLogUser(report_log_options, HighsLogType::kWarning,
+  highsLogUser(report_log_options, HighsLogType::kError,
                "Value \"%s\" for %s option is not one of \"%s\" or \"%s\"\n",
                value.c_str(), name.c_str(), kHighsOffString.c_str(),
                kHighsOnString.c_str());
   return false;
 }
 
-bool commandLineSolverOk(const HighsLogOptions& report_log_options,
-                         const string& value) {
-  if (value == kSimplexString || value == kHighsChooseString ||
-      value == kIpmString || value == kPdlpString)
+bool optionSolverOk(const HighsLogOptions& report_log_options,
+                    const string& value) {
+#ifndef HIPO
+  if (value == kHipoString) {
+    highsLogUser(
+        report_log_options, HighsLogType::kError,
+        "The HiPO solver was requested via the \"%s\" option, but this build "
+        "was compiled without HiPO support. Reconfigure with FAST_BUILD=ON "
+        "and -DHIPO=ON to enable HiPO.\n",
+        kSolverString.c_str());
+    return false;
+  }
+#endif
+  if (value == kHighsChooseString || value == kSimplexString ||
+      value == kIpmString ||
+#ifdef HIPO
+      value == kHipoString ||
+#endif
+      value == kIpxString || value == kPdlpString)
     return true;
-  highsLogUser(report_log_options, HighsLogType::kWarning,
-               "Value \"%s\" for solver option is not one of \"%s\", \"%s\", "
-               "\"%s\" or \"%s\"\n",
-               value.c_str(), kSimplexString.c_str(),
+  highsLogUser(report_log_options, HighsLogType::kError,
+               "Value \"%s\" for LP solver option (\"%s\") is not one of "
+#ifdef HIPO
+               "\"%s\", "
+#endif
+               "\"%s\", \"%s\", \"%s\", \"%s\" or \"%s\"\n",
+               value.c_str(), kSolverString.c_str(), kHighsChooseString.c_str(),
+               kSimplexString.c_str(), kIpmString.c_str(),
+#ifdef HIPO
+               kHipoString.c_str(),
+#endif
+               kIpxString.c_str(), kPdlpString.c_str());
+  return false;
+}
+
+bool optionMipLpSolverOk(const HighsLogOptions& report_log_options,
+                         const string& value) {
+#ifndef HIPO
+  if (value == kHipoString) {
+    highsLogUser(
+        report_log_options, HighsLogType::kError,
+        "The HiPO solver was requested via the \"%s\" option, but this build "
+        "was compiled without HiPO support. Reconfigure with FAST_BUILD=ON "
+        "and -DHIPO=ON to enable HiPO.\n",
+        kMipLpSolverString.c_str());
+    return false;
+  }
+#endif
+  if (value == kHighsChooseString || value == kSimplexString ||
+      value == kIpmString ||
+#ifdef HIPO
+      value == kHipoString ||
+#endif
+      value == kIpxString)
+    return true;
+  highsLogUser(report_log_options, HighsLogType::kError,
+               "Value \"%s\" for MIP LP solver option (\"%s\") is not one of "
+#ifdef HIPO
+               "\"%s\", "
+#endif
+               "\"%s\", \"%s\", \"%s\" or \"%s\"\n",
+               value.c_str(), kMipLpSolverString.c_str(),
+               kHighsChooseString.c_str(), kSimplexString.c_str(),
+               kIpmString.c_str(),
+#ifdef HIPO
+               kHipoString.c_str(),
+#endif
+               kIpxString.c_str());
+  return false;
+}
+
+bool optionMipIpmSolverOk(const HighsLogOptions& report_log_options,
+                          const string& value) {
+#ifndef HIPO
+  if (value == kHipoString) {
+    highsLogUser(
+        report_log_options, HighsLogType::kError,
+        "The HiPO solver was requested via the \"%s\" option, but this build "
+        "was compiled without HiPO support. Reconfigure with FAST_BUILD=ON "
+        "and -DHIPO=ON to enable HiPO.\n",
+        kMipIpmSolverString.c_str());
+    return false;
+  }
+#endif
+  if (value == kHighsChooseString || value == kIpmString ||
+#ifdef HIPO
+      value == kHipoString ||
+#endif
+      value == kIpxString)
+    return true;
+  highsLogUser(report_log_options, HighsLogType::kError,
+               "Value \"%s\" for MIP IPM solver (\"%s\") option is not one of "
+#ifdef HIPO
+               "\"%s\", "
+#endif
+               "\"%s\", \"%s\" or \"%s\"\n",
+               value.c_str(), kMipIpmSolverString.c_str(),
                kHighsChooseString.c_str(), kIpmString.c_str(),
-               kPdlpString.c_str());
+#ifdef HIPO
+               kHipoString.c_str(),
+#endif
+               kIpxString.c_str());
   return false;
 }
 
@@ -309,14 +402,14 @@ OptionStatus checkOption(const HighsLogOptions& report_log_options,
 OptionStatus checkOptionValue(const HighsLogOptions& report_log_options,
                               OptionRecordInt& option, const HighsInt value) {
   if (value < option.lower_bound) {
-    highsLogUser(report_log_options, HighsLogType::kWarning,
+    highsLogUser(report_log_options, HighsLogType::kError,
                  "checkOptionValue: Value %" HIGHSINT_FORMAT
                  " for option \"%s\" is below "
                  "lower bound of %" HIGHSINT_FORMAT "\n",
                  value, option.name.c_str(), option.lower_bound);
     return OptionStatus::kIllegalValue;
   } else if (value > option.upper_bound) {
-    highsLogUser(report_log_options, HighsLogType::kWarning,
+    highsLogUser(report_log_options, HighsLogType::kError,
                  "checkOptionValue: Value %" HIGHSINT_FORMAT
                  " for option \"%s\" is above "
                  "upper bound of %" HIGHSINT_FORMAT "\n",
@@ -329,13 +422,13 @@ OptionStatus checkOptionValue(const HighsLogOptions& report_log_options,
 OptionStatus checkOptionValue(const HighsLogOptions& report_log_options,
                               OptionRecordDouble& option, const double value) {
   if (value < option.lower_bound) {
-    highsLogUser(report_log_options, HighsLogType::kWarning,
+    highsLogUser(report_log_options, HighsLogType::kError,
                  "checkOptionValue: Value %g for option \"%s\" is below "
                  "lower bound of %g\n",
                  value, option.name.c_str(), option.lower_bound);
     return OptionStatus::kIllegalValue;
   } else if (value > option.upper_bound) {
-    highsLogUser(report_log_options, HighsLogType::kWarning,
+    highsLogUser(report_log_options, HighsLogType::kError,
                  "checkOptionValue: Value %g for option \"%s\" is above "
                  "upper bound of %g\n",
                  value, option.name.c_str(), option.upper_bound);
@@ -350,20 +443,26 @@ OptionStatus checkOptionValue(const HighsLogOptions& report_log_options,
   // Setting a string option. For some options only particular values
   // are permitted, so check them
   if (option.name == kPresolveString) {
-    if (!commandLineOffChooseOnOk(report_log_options, option.name, value) &&
+    if (!optionOffChooseOnOk(report_log_options, option.name, value) &&
         value != "mip")
       return OptionStatus::kIllegalValue;
   } else if (option.name == kSolverString) {
-    if (!commandLineSolverOk(report_log_options, value))
+    if (!optionSolverOk(report_log_options, value))
+      return OptionStatus::kIllegalValue;
+  } else if (option.name == kMipLpSolverString) {
+    if (!optionMipLpSolverOk(report_log_options, value))
+      return OptionStatus::kIllegalValue;
+  } else if (option.name == kMipIpmSolverString) {
+    if (!optionMipIpmSolverOk(report_log_options, value))
       return OptionStatus::kIllegalValue;
   } else if (option.name == kParallelString) {
-    if (!commandLineOffChooseOnOk(report_log_options, option.name, value))
+    if (!optionOffChooseOnOk(report_log_options, option.name, value))
       return OptionStatus::kIllegalValue;
   } else if (option.name == kRunCrossoverString) {
-    if (!commandLineOffChooseOnOk(report_log_options, option.name, value))
+    if (!optionOffChooseOnOk(report_log_options, option.name, value))
       return OptionStatus::kIllegalValue;
   } else if (option.name == kRangingString) {
-    if (!commandLineOffOnOk(report_log_options, option.name, value))
+    if (!optionOffOnOk(report_log_options, option.name, value))
       return OptionStatus::kIllegalValue;
   }
   return OptionStatus::kOk;
@@ -374,8 +473,6 @@ OptionStatus setLocalOptionValue(const HighsLogOptions& report_log_options,
                                  std::vector<OptionRecord*>& option_records,
                                  const bool value) {
   HighsInt index;
-  //  Rprintf("setLocalOptionValue: \"%s\" with bool %" HIGHSINT_FORMAT "\n",
-  //  name.c_str(), value);
   OptionStatus status =
       getOptionIndex(report_log_options, name, option_records, index);
   if (status != OptionStatus::kOk) return status;
@@ -396,8 +493,6 @@ OptionStatus setLocalOptionValue(const HighsLogOptions& report_log_options,
                                  std::vector<OptionRecord*>& option_records,
                                  const HighsInt value) {
   HighsInt index;
-  //  Rprintf("setLocalOptionValue: \"%s\" with HighsInt %" HIGHSINT_FORMAT "\n",
-  //  name.c_str(), value);
   OptionStatus status =
       getOptionIndex(report_log_options, name, option_records, index);
   if (status != OptionStatus::kOk) return status;
@@ -425,8 +520,6 @@ OptionStatus setLocalOptionValue(const HighsLogOptions& report_log_options,
                                  std::vector<OptionRecord*>& option_records,
                                  const double value) {
   HighsInt index;
-  //  Rprintf("setLocalOptionValue: \"%s\" with double %g\n", name.c_str(),
-  //  value);
   OptionStatus status =
       getOptionIndex(report_log_options, name, option_records, index);
   if (status != OptionStatus::kOk) return status;
