@@ -313,16 +313,17 @@ void HighsCliqueTable::doAddClique(const CliqueVar* cliquevars,
   cliques[cliqueid].equality = equality;
   cliques[cliqueid].origin = origin;
 
-  std::set<std::pair<HighsInt, int>>::iterator it;
+  decltype(freespaces)::iterator it;
   HighsInt maxEnd;
-  if (freespaces.empty() || (it = freespaces.lower_bound(std::make_pair(
-                                 numcliquevars, -1))) == freespaces.end()) {
+  if (freespaces.empty() ||
+      (it = freespaces.lower_bound(
+           std::make_pair(numcliquevars, HighsInt{-1}))) == freespaces.end()) {
     cliques[cliqueid].start = cliqueentries.size();
     cliques[cliqueid].end = cliques[cliqueid].start + numcliquevars;
     maxEnd = cliques[cliqueid].end;
     cliqueentries.resize(cliques[cliqueid].end);
   } else {
-    std::pair<HighsInt, int> freespace = *it;
+    auto freespace = *it;
     freespaces.erase(it);
 
     cliques[cliqueid].start = freespace.second;
@@ -861,6 +862,7 @@ void HighsCliqueTable::extractCliques(
     for (HighsInt i = 0; i != nbin; ++i) {
       HighsInt bincol = inds[perm[i]];
       HighsCDouble impliedub = HighsCDouble(rhs) - vals[perm[i]];
+      if (implics.tooManyVarBounds()) break;
       for (HighsInt j = nbin; j != ntotal; ++j) {
         HighsInt col = inds[perm[j]];
         if (globaldom.isFixed(col)) continue;
@@ -963,7 +965,7 @@ void HighsCliqueTable::extractCliques(
       // if (clique.size() > 2) runCliqueSubsumption(globaldom, clique);
       // runCliqueMerging(globaldom, clique);
       // if (clique.size() >= 2) {
-      addClique(mipsolver, clique.data(), clique.size());
+      addClique(mipsolver, clique.data(), static_cast<HighsInt>(clique.size()));
       if (globaldom.infeasible()) return;
       //}
     }
@@ -1106,6 +1108,10 @@ void HighsCliqueTable::extractCliquesFromCut(const HighsMipSolver& mipsolver,
     }
   }
 
+  if (rhs - minact < 0.0) {
+    minact = rhs;
+  }
+
   for (HighsInt i = 0; i != len; ++i) {
     if (mipsolver.variableType(inds[i]) == HighsVarType::kContinuous) continue;
 
@@ -1256,7 +1262,7 @@ void HighsCliqueTable::extractCliquesFromCut(const HighsMipSolver& mipsolver,
       // Rprintf("extracted clique from cut\n");
       // if (clique.size() > 2) runCliqueSubsumption(globaldom, clique);
 
-      addClique(mipsolver, clique.data(), clique.size());
+      addClique(mipsolver, clique.data(), static_cast<HighsInt>(clique.size()));
       if (globaldom.infeasible() || numEntries >= maxNewEntries) return;
     }
 
@@ -1315,7 +1321,8 @@ void HighsCliqueTable::extractCliques(HighsMipSolver& mipsolver,
 
       if (issetppc) {
         bool equality = mipsolver.rowLower(i) == 1.0;
-        addClique(mipsolver, clique.data(), clique.size(), equality, i);
+        addClique(mipsolver, clique.data(),
+                  static_cast<HighsInt>(clique.size()), equality, i);
         if (globaldom.infeasible()) return;
         continue;
       }
@@ -1506,7 +1513,7 @@ void HighsCliqueTable::extractObjCliques(HighsMipSolver& mipsolver) {
       // Rprintf("extracted clique from obj\n");
       // if (clique.size() > 2) runCliqueSubsumption(globaldom, clique);
 
-      addClique(mipsolver, clique.data(), clique.size());
+      addClique(mipsolver, clique.data(), static_cast<HighsInt>(clique.size()));
       if (globaldom.infeasible()) return;
     }
 
