@@ -396,6 +396,7 @@ void HighsImplications::addVUB(HighsInt col, HighsInt vubcol, double vubcoef,
   // assume that VUBs do not have infinite coefficients and infinite constant
   // terms since such VUBs effectively evaluate to NaN.
   assert(std::abs(vubcoef) != kHighsInf || std::abs(vubconstant) != kHighsInf);
+  if (tooManyVarBounds()) return;
 
   VarBound vub{vubcoef, vubconstant};
 
@@ -415,7 +416,8 @@ void HighsImplications::addVUB(HighsInt col, HighsInt vubcol, double vubcoef,
       currentvub.coef = vubcoef;
       currentvub.constant = vubconstant;
     }
-  }
+  } else
+    numVarBounds++;
 }
 
 void HighsImplications::addVLB(HighsInt col, HighsInt vlbcol, double vlbcoef,
@@ -423,6 +425,7 @@ void HighsImplications::addVLB(HighsInt col, HighsInt vlbcol, double vlbcoef,
   // assume that VLBs do not have infinite coefficients and infinite constant
   // terms since such VLBs effectively evaluate to NaN.
   assert(std::abs(vlbcoef) != kHighsInf || std::abs(vlbconstant) != kHighsInf);
+  if (tooManyVarBounds()) return;
 
   VarBound vlb{vlbcoef, vlbconstant};
 
@@ -443,7 +446,8 @@ void HighsImplications::addVLB(HighsInt col, HighsInt vlbcol, double vlbcoef,
       currentvlb.coef = vlbcoef;
       currentvlb.constant = vlbconstant;
     }
-  }
+  } else
+    numVarBounds++;
 }
 
 void HighsImplications::rebuild(HighsInt ncols,
@@ -470,6 +474,7 @@ void HighsImplications::rebuild(HighsInt ncols,
   vlbs.shrink_to_fit();
   vlbs.resize(ncols);
   numImplications = 0;
+  numVarBounds = 0;
   HighsInt oldncols = oldvubs.size();
 
   nextCleanupCall = mipsolver.numNonzero();
@@ -636,7 +641,7 @@ void HighsImplications::separateImpliedBounds(
           vals[0] = -1.0;
           inds[0] = implics[i].column;
           vals[1] =
-              globaldomain.col_lower_[implics[i].column] - implics[i].boundval;
+              implics[i].boundval - globaldomain.col_lower_[implics[i].column];
           inds[1] = col;
           rhs = -globaldomain.col_lower_[implics[i].column];
         }
@@ -711,6 +716,8 @@ void HighsImplications::cleanupVarbounds(HighsInt col) {
   double lb = mipsolver.mipdata_->domain.col_lower_[col];
 
   if (ub == lb) {
+    numVarBounds -= vlbs.size();
+    numVarBounds -= vubs.size();
     vlbs[col].clear();
     vubs[col].clear();
     return;
@@ -726,10 +733,9 @@ void HighsImplications::cleanupVarbounds(HighsInt col) {
     if (infeasible) return;
   });
 
-  if (!delVbds.empty()) {
-    for (HighsInt vubCol : delVbds) vubs[col].erase(vubCol);
-    delVbds.clear();
-  }
+  for (HighsInt vubCol : delVbds) vubs[col].erase(vubCol);
+  numVarBounds -= delVbds.size();
+  delVbds.clear();
 
   vlbs[col].for_each([&](HighsInt vlbCol, VarBound& vlb) {
     bool redundant = false;
@@ -740,6 +746,7 @@ void HighsImplications::cleanupVarbounds(HighsInt col) {
   });
 
   for (HighsInt vlbCol : delVbds) vlbs[col].erase(vlbCol);
+  numVarBounds -= delVbds.size();
 }
 
 void HighsImplications::cleanupVlb(HighsInt col, HighsInt vlbCol,
