@@ -43,12 +43,16 @@ HighsMipSolver::HighsMipSolver(HighsCallback& callback,
       pscostinit(nullptr),
       clqtableinit(nullptr),
       implicinit(nullptr) {
+  timer_.setPrintfFlag(options_mip_->output_flag, options_mip_->log_to_console);
   assert(!submip || submip_level > 0);
   max_submip_level = 0;
+  // Initialise empty terminator
+  initialiseTerminator();
+  assert(termination_status_ == HighsModelStatus::kNotset);
   if (solution.value_valid) {
+#ifndef NDEBUG
     // MIP solver doesn't check row residuals, but they should be OK
     // so validate using assert
-#ifndef NDEBUG
     bool valid, integral, feasible;
     assessLpPrimalSolution("For debugging: ", options, lp, solution, valid,
                            integral, feasible);
@@ -74,6 +78,7 @@ void HighsMipSolver::run() {
     analysis_.analyse_mip_time = false;
   } else {
     analysis_.timer_ = &this->timer_;
+    analysis_.sub_solver_call_time_ = &this->sub_solver_call_time_;
     analysis_.setup(*orig_model_, *options_mip_);
   }
   timer_.start();
@@ -88,10 +93,16 @@ void HighsMipSolver::run() {
   analysis_.mipTimerStart(kMipClockInit);
   mipdata_->init();
   analysis_.mipTimerStop(kMipClockInit);
+#ifdef HIGHS_DEBUGSOL
+  mipdata_->debugSolution.activate();
+  bool debugSolActive = false;
+  std::swap(mipdata_->debugSolution.debugSolActive, debugSolActive);
+#endif
   analysis_.mipTimerStart(kMipClockRunPresolve);
-  mipdata_->runPresolve(options_mip_->presolve_reduction_limit);
+  mipdata_->runMipPresolve(options_mip_->presolve_reduction_limit);
   analysis_.mipTimerStop(kMipClockRunPresolve);
   analysis_.mipTimerStop(kMipClockPresolve);
+
   if (analysis_.analyse_mip_time && !submip)
     highsLogUser(options_mip_->log_options, HighsLogType::kInfo,
                  "MIP-Timing: %11.2g - completed presolve\n", timer_.read());
@@ -120,6 +131,9 @@ void HighsMipSolver::run() {
     highsLogUser(options_mip_->log_options, HighsLogType::kInfo,
                  "MIP-Timing: %11.2g - starting  setup\n", timer_.read());
   analysis_.mipTimerStart(kMipClockRunSetup);
+#ifdef HIGHS_DEBUGSOL
+  mipdata_->debugSolution.debugSolActive = debugSolActive;
+#endif
   mipdata_->runSetup();
   analysis_.mipTimerStop(kMipClockRunSetup);
   if (analysis_.analyse_mip_time && !submip)
@@ -132,35 +146,54 @@ restart:
       cleanupSolve();
       return;
     }
-    // Possibly look for primal solution from the user
-    if (!submip && callback_->user_callback &&
-        callback_->active[kCallbackMipUserSolution])
-      mipdata_->callbackUserSolution(solution_objective_,
-                                     kUserMipSolutionCallbackOriginAfterSetup);
+    // Possibly query existence of an external solution
+    if (!submip)
+      mipdata_->queryExternalSolution(
+          solution_objective_, kExternalMipSolutionQueryOriginAfterSetup);
 
     // Apply the trivial heuristics
     analysis_.mipTimerStart(kMipClockTrivialHeuristics);
-    HighsModelStatus model_status = mipdata_->trivialHeuristics();
+    HighsModelStatus returned_model_status = mipdata_->trivialHeuristics();
     analysis_.mipTimerStop(kMipClockTrivialHeuristics);
     if (modelstatus_ == HighsModelStatus::kNotset &&
-        model_status == HighsModelStatus::kInfeasible) {
+        returned_model_status == HighsModelStatus::kInfeasible) {
       // trivialHeuristics can spot trivial infeasibility, so act on it
-      modelstatus_ = model_status;
+      modelstatus_ = returned_model_status;
       cleanupSolve();
       return;
     }
+    // Apply the feasibility jump heuristic (if enabled)
+    if (options_mip_->mip_heuristic_run_feasibility_jump) {
+      analysis_.mipTimerStart(kMipClockFeasibilityJump);
+      HighsModelStatus returned_model_status = mipdata_->feasibilityJump();
+      analysis_.mipTimerStop(kMipClockFeasibilityJump);
+      if (modelstatus_ == HighsModelStatus::kNotset &&
+          returned_model_status == HighsModelStatus::kInfeasible) {
+        // feasibilityJump can spot trivial infeasibility, so act on it
+        modelstatus_ = returned_model_status;
+        cleanupSolve();
+        return;
+      }
+    }
+    // End of pre-root-node heuristics
     if (analysis_.analyse_mip_time && !submip)
-      highsLogUser(options_mip_->log_options, HighsLogType::kInfo,
-                   "MIP-Timing: %11.2g - starting evaluate root node\n",
-                   timer_.read());
+      if (analysis_.analyse_mip_time & !submip)
+        highsLogUser(options_mip_->log_options, HighsLogType::kInfo,
+                     "MIP-Timing: %11.2g - starting evaluate root node\n",
+                     timer_.read());
     analysis_.mipTimerStart(kMipClockEvaluateRootNode);
     mipdata_->evaluateRootNode();
     analysis_.mipTimerStop(kMipClockEvaluateRootNode);
+    if (this->terminate()) {
+      modelstatus_ = this->terminationStatus();
+      cleanupSolve();
+      return;
+    }
     // Sometimes the analytic centre calculation is not completed when
     // evaluateRootNode returns, so stop its clock if it's running
     if (analysis_.analyse_mip_time &&
-        analysis_.mipTimerRunning(kMipClockIpmSolveLp))
-      analysis_.mipTimerStop(kMipClockIpmSolveLp);
+        analysis_.mipTimerRunning(kMipClockIpxSolveLp))
+      analysis_.mipTimerStop(kMipClockIpxSolveLp);
     if (analysis_.analyse_mip_time && !submip)
       highsLogUser(options_mip_->log_options, HighsLogType::kInfo,
                    "MIP-Timing: %11.2g - completed evaluate root node\n",
@@ -211,11 +244,10 @@ restart:
   double lowerBoundLastCheck = mipdata_->lower_bound;
   analysis_.mipTimerStart(kMipClockSearch);
   while (search.hasNode()) {
-    // Possibly look for primal solution from the user
-    if (!submip && callback_->user_callback &&
-        callback_->active[kCallbackMipUserSolution])
-      mipdata_->callbackUserSolution(solution_objective_,
-                                     kUserMipSolutionCallbackOriginBeforeDive);
+    // Possibly query existence of an external solution
+    if (!submip)
+      mipdata_->queryExternalSolution(
+          solution_objective_, kExternalMipSolutionQueryOriginBeforeDive);
 
     analysis_.mipTimerStart(kMipClockPerformAging1);
     mipdata_->conflictPool.performAging();
@@ -275,6 +307,10 @@ restart:
 
           mipdata_->heuristics.flushStatistics();
           analysis_.mipTimerStop(kMipClockDivePrimalHeuristics);
+          if (mipdata_->terminatorTerminated()) {
+            cleanupSolve();
+            return;
+          }
         }
       }
 
@@ -667,6 +703,22 @@ restart:
 }
 
 void HighsMipSolver::cleanupSolve() {
+  if (mipdata_->terminatorActive()) {
+    if (mipdata_->terminatorTerminated()) {
+      // Indicate that this instance has been interrupted
+      modelstatus_ = HighsModelStatus::kHighsInterrupt;
+    } else if (!submip) {
+      // When sub-MIPs call cleanupSolve(), they generally don't have
+      // a termination criterion for the whole MIP solver
+      //
+      // Possibly allow sub-MIPs to terminate if the time limit is
+      // reached
+      //
+      // No other instance has terminated, so terminate
+      mipdata_->terminatorTerminate();
+    }
+  }
+
   // Force a final logging line
   mipdata_->printDisplayLine(kSolutionSourceCleanup);
   // Stop the solve clock - which won't be running if presolve
@@ -727,10 +779,12 @@ void HighsMipSolver::cleanupSolve() {
   std::string solutionstatus = "-";
 
   if (havesolution) {
-    bool feasible =
+    // Surely this definition of feasible is unnecessary
+    bool lc_feasible =
         bound_violation_ <= options_mip_->mip_feasibility_tolerance &&
         integrality_violation_ <= options_mip_->mip_feasibility_tolerance &&
         row_violation_ <= options_mip_->mip_feasibility_tolerance;
+    assert(feasible == lc_feasible);
     solutionstatus = feasible ? "feasible" : "infeasible";
   }
 
@@ -742,35 +796,8 @@ void HighsMipSolver::cleanupSolve() {
   else
     gap_ = kHighsInf;
 
-  std::array<char, 128> gapString = {};
-
-  if (gap_ == kHighsInf)
-    std::strcpy(gapString.data(), "inf");
-  else {
-    double printTol = std::max(std::min(1e-2, 1e-1 * gap_), 1e-6);
-    auto gapValString = highsDoubleToString(100.0 * gap_, printTol);
-    double gapTol = options_mip_->mip_rel_gap;
-
-    if (options_mip_->mip_abs_gap > options_mip_->mip_feasibility_tolerance) {
-      gapTol = primal_bound_ == 0.0
-                   ? kHighsInf
-                   : std::max(gapTol,
-                              options_mip_->mip_abs_gap / fabs(primal_bound_));
-    }
-
-    if (gapTol == 0.0)
-      std::snprintf(gapString.data(), gapString.size(), "%s%%",
-                    gapValString.data());
-    else if (gapTol != kHighsInf) {
-      printTol = std::max(std::min(1e-2, 1e-1 * gapTol), 1e-6);
-      auto gapTolString = highsDoubleToString(100.0 * gapTol, printTol);
-      std::snprintf(gapString.data(), gapString.size(),
-                    "%s%% (tolerance: %s%%)", gapValString.data(),
-                    gapTolString.data());
-    } else
-      std::snprintf(gapString.data(), gapString.size(), "%s%% (tolerance: inf)",
-                    gapValString.data());
-  }
+  std::array<char, 128> gapString =
+      getGapString(gap_, primal_bound_, options_mip_);
 
   bool timeless_log = options_mip_->timeless_log;
   highsLogUser(options_mip_->log_options, HighsLogType::kInfo,
@@ -800,42 +827,58 @@ void HighsMipSolver::cleanupSolve() {
                  "                    %.12g (row viol.)\n",
                  solution_objective_, bound_violation_, integrality_violation_,
                  row_violation_);
-  if (!timeless_log)
+  if (!timeless_log) {
     highsLogUser(options_mip_->log_options, HighsLogType::kInfo,
-                 "  Timing            %.2f (total)\n"
-                 "                    %.2f (presolve)\n"
-                 "                    %.2f (solve)\n"
-                 "                    %.2f (postsolve)\n",
-                 timer_.read(), analysis_.mipTimerRead(kMipClockPresolve),
-                 analysis_.mipTimerRead(kMipClockSolve),
-                 analysis_.mipTimerRead(kMipClockPostsolve));
+                 "  Timing            %.2f\n", timer_.read());
+    if (analysis_.analyse_mip_time)
+      highsLogUser(options_mip_->log_options, HighsLogType::kInfo,
+                   "                    %.2f (presolve)\n"
+                   "                    %.2f (solve)\n"
+                   "                    %.2f (postsolve)\n",
+                   analysis_.mipTimerRead(kMipClockPresolve),
+                   analysis_.mipTimerRead(kMipClockSolve),
+                   analysis_.mipTimerRead(kMipClockPostsolve));
+  }
   highsLogUser(options_mip_->log_options, HighsLogType::kInfo,
                "  Max sub-MIP depth %d\n"
-               "  Nodes             %llu\n"
-               "  Repair LPs        %llu (%llu feasible; %llu iterations)\n"
-               "  LP iterations     %llu (total)\n"
-               "                    %llu (strong br.)\n"
-               "                    %llu (separation)\n"
-               "                    %llu (heuristics)\n",
-               int(max_submip_level), (long long unsigned)mipdata_->num_nodes,
-               (long long unsigned)mipdata_->total_repair_lp,
-               (long long unsigned)mipdata_->total_repair_lp_feasible,
-               (long long unsigned)mipdata_->total_repair_lp_iterations,
-               (long long unsigned)mipdata_->total_lp_iterations,
-               (long long unsigned)mipdata_->sb_lp_iterations,
-               (long long unsigned)mipdata_->sepa_lp_iterations,
-               (long long unsigned)mipdata_->heuristic_lp_iterations);
+               "  Nodes             %llu\n",
+               int(max_submip_level), (long long unsigned)mipdata_->num_nodes);
+  if (mipdata_->total_repair_lp) {
+    highsLogUser(options_mip_->log_options, HighsLogType::kInfo,
+                 "  Repair LPs        %llu (%llu feasible; %llu iterations)\n",
+                 (long long unsigned)mipdata_->total_repair_lp,
+                 (long long unsigned)mipdata_->total_repair_lp_feasible,
+                 (long long unsigned)mipdata_->total_repair_lp_iterations);
+  } else {
+    highsLogUser(options_mip_->log_options, HighsLogType::kInfo,
+                 "  Repair LPs        0\n");
+  }
+  highsLogUser(options_mip_->log_options, HighsLogType::kInfo,
+               "  LP iterations     %llu\n",
+               (long long unsigned)mipdata_->total_lp_iterations);
+  if (mipdata_->total_lp_iterations)
+    highsLogUser(options_mip_->log_options, HighsLogType::kInfo,
+                 "                    %llu (strong br.)\n"
+                 "                    %llu (separation)\n"
+                 "                    %llu (heuristics)\n",
+                 (long long unsigned)mipdata_->sb_lp_iterations,
+                 (long long unsigned)mipdata_->sepa_lp_iterations,
+                 (long long unsigned)mipdata_->heuristic_lp_iterations);
 
   if (!timeless_log) analysis_.reportMipTimer();
 
+  analysis_.checkSubSolverCallTime(sub_solver_call_time_);
+
   assert(modelstatus_ != HighsModelStatus::kNotset);
+
+  if (improving_solution_file_ != nullptr) fclose(improving_solution_file_);
 }
 
 // Only called in Highs::runPresolve
-void HighsMipSolver::runPresolve(const HighsInt presolve_reduction_limit) {
+void HighsMipSolver::runMipPresolve(const HighsInt presolve_reduction_limit) {
   mipdata_ = decltype(mipdata_)(new HighsMipSolverData(*this));
   mipdata_->init();
-  mipdata_->runPresolve(presolve_reduction_limit);
+  mipdata_->runMipPresolve(presolve_reduction_limit);
 }
 
 const HighsLp& HighsMipSolver::getPresolvedModel() const {
@@ -853,30 +896,65 @@ presolve::HighsPostsolveStack HighsMipSolver::getPostsolveStack() const {
 void HighsMipSolver::callbackGetCutPool() const {
   assert(callback_->user_callback);
   assert(callback_->callbackActive(kCallbackMipGetCutPool));
-  HighsCallbackDataOut& data_out = callback_->data_out;
+  HighsCallbackOutput& data_out = callback_->data_out;
 
-  std::vector<double> cut_lower;
-  std::vector<double> cut_upper;
   HighsSparseMatrix cut_matrix;
-
   mipdata_->lp.getCutPool(data_out.cutpool_num_col, data_out.cutpool_num_cut,
-                          cut_lower, cut_upper, cut_matrix);
+                          data_out.cutpool_lower, data_out.cutpool_upper,
+                          cut_matrix);
+
+  // take ownership
+  data_out.cutpool_start = std::move(cut_matrix.start_);
+  data_out.cutpool_index = std::move(cut_matrix.index_);
+  data_out.cutpool_value = std::move(cut_matrix.value_);
 
-  data_out.cutpool_num_nz = cut_matrix.numNz();
-  data_out.cutpool_start = cut_matrix.start_.data();
-  data_out.cutpool_index = cut_matrix.index_.data();
-  data_out.cutpool_value = cut_matrix.value_.data();
-  data_out.cutpool_lower = cut_lower.data();
-  data_out.cutpool_upper = cut_upper.data();
   callback_->user_callback(kCallbackMipGetCutPool, "MIP cut pool",
                            &callback_->data_out, &callback_->data_in,
                            callback_->user_callback_data);
 }
 
-bool HighsMipSolver::solutionFeasible(
-    const HighsLp* lp, const std::vector<double>& col_value,
-    const std::vector<double>* pass_row_value, double& bound_violation,
-    double& row_violation, double& integrality_violation, HighsCDouble& obj) {
+std::array<char, 128> getGapString(const double gap_,
+                                   const double primal_bound_,
+                                   const HighsOptions* options_mip_) {
+  std::array<char, 128> gapString = {};
+  if (gap_ == kHighsInf)
+    std::strcpy(gapString.data(), "inf");
+  else {
+    double printTol = std::max(std::min(1e-2, 1e-1 * gap_), 1e-6);
+    auto gapValString = highsDoubleToString(100.0 * gap_, printTol);
+    double gapTol = options_mip_->mip_rel_gap;
+
+    if (options_mip_->mip_abs_gap > options_mip_->mip_feasibility_tolerance) {
+      gapTol = primal_bound_ == 0.0
+                   ? kHighsInf
+                   : std::max(gapTol,
+                              options_mip_->mip_abs_gap / fabs(primal_bound_));
+    }
+
+    if (gapTol == 0.0)
+      std::snprintf(gapString.data(), gapString.size(), "%s%%",
+                    gapValString.data());
+    else if (gapTol != kHighsInf) {
+      printTol = std::max(std::min(1e-2, 1e-1 * gapTol), 1e-6);
+      auto gapTolString = highsDoubleToString(100.0 * gapTol, printTol);
+      std::snprintf(gapString.data(), gapString.size(),
+                    "%s%% (tolerance: %s%%)", gapValString.data(),
+                    gapTolString.data());
+    } else
+      std::snprintf(gapString.data(), gapString.size(), "%s%% (tolerance: inf)",
+                    gapValString.data());
+  }
+
+  return gapString;
+}
+
+bool HighsMipSolver::solutionFeasible(const HighsLp* lp,
+                                      const std::vector<double>& col_value,
+                                      const std::vector<double>* pass_row_value,
+                                      double& bound_violation,
+                                      double& row_violation,
+                                      double& integrality_violation,
+                                      HighsCDouble& obj) const {
   bound_violation = 0;
   row_violation = 0;
   integrality_violation = 0;
@@ -947,3 +1025,26 @@ bool HighsMipSolver::solutionFeasible(
                         row_violation <= mip_feasibility_tolerance;
   return feasible;
 }
+
+std::vector<HighsModelStatus> HighsMipSolver::initialiseTerminatorRecord(
+    HighsInt num_instance) const {
+  std::vector<HighsModelStatus> record(num_instance, HighsModelStatus::kNotset);
+  return record;
+}
+
+void HighsMipSolver::initialiseTerminator(HighsInt num_instance_,
+                                          HighsInt my_instance_,
+                                          HighsModelStatus* record_) {
+  this->termination_status_ = HighsModelStatus::kNotset;
+  this->terminator_.clear();
+  this->terminator_.initialise(num_instance_, my_instance_, record_);
+}
+
+void HighsMipSolver::initialiseTerminator(const HighsMipSolver& mip_solver) {
+  this->terminator_.clear();
+  if (!mip_solver.mipdata_->terminatorActive()) return;
+  assert(mip_solver.mipdata_->terminatorConcurrency() > 0);
+  this->initialiseTerminator(mip_solver.mipdata_->terminatorConcurrency(),
+                             mip_solver.mipdata_->terminatorMyInstance(),
+                             mip_solver.terminator_.record);
+}
